from .colorPrint import *
from .typewritePrint import *; from .reversePrint import *; from .asciiPrint import *
from .informUserPrint import *; from .warnUserPrint import *; from .errorUserPrint import *


def reversePrint(text: str) -> str:
    string_ = text
    final_string = ""

    def reverse(string_1: str):
        if not string_1:
            return ""
        else:
            front_part = reverse(string_1[1:])
            back_part = string_1[0]
            front_part + back_part + string_[:-len(string_1)]

        return front_part + back_part[0]

    reverse_text = reverse(string_)
    reverse_text = reverse_text.split()

    for index in reverse_text:
        final_string += f"{index} "
    print(final_string)
    


def typewritePrint(text: str):
    """
    
    Usage
    -----
    Acts like a normal print statement but with a typewriter effect.

    Parameters
    ----------
    text : string
        The text you want to be typed out.
    
    Returns
    -------
        Returns out the finished product into the terminal.
        
    """
    import sys, time

    for letter in text:
        time.sleep(0.03)
        sys.stdout.write(letter)
        sys.stdout.flush()
    print("")



def colorPrint(text, color: str=None, style: str=None, bg_color: str=None) -> str:
    """

    Usage
    -----
    Acts like a normal print statement but with customizable features.\n
    Can be customized in many ways.

    Parameters
    ----------
    text : string
        The text you want to customize.
    color : string
        The color you want the text to be. -/- Gray, Red, Green, Yellow, Blue, Purple, Cyan, White or Rainbow.
    style : string
        The text style you want the text to be. -/- Bold, Dimmed, Italicized or Underlined.
    bg_color : string
        The background color you want behind the text to be. -/- Gray, Red, Green, Yellow, Blue, Purple, Cyan or White.

    Returns
    -------
        Returns out the finished product into the terminal.
        
    """

    colors = ["Gray", "Red", "Green", "Yellow", "Blue", "Purple", "Cyan", "White"]
    styles = ["Bold", "Dim", "Italicize", "Underline"]
    colorcodes = ["\033[0;30;48m", "\033[0;31;48m", "\033[0;32;48m", "\033[0;33;48m", "\033[0;34;48m", "\033[0;35;48m", "\033[0;36;48m", "\033[0;37;48m"]
    rbcolors = ["\033[0;31;48m", "\033[0;32;48m", "\033[0;33;48m", "\033[0;34;48m", "\033[0;35;48m", "\033[0;36;48m", "\033[0;37;48m"]
    endcode = "\033[0m"
    code = "\033[0;37;40m"

    if color != None:
        if color.capitalize() == "Rainbow":
            try:
                newtxt = ""
                rainbow_index = 0
                txt = list(text)
                for char in txt:
                    newtxt += rbcolors[rainbow_index] + char + endcode
                    rainbow_index += 1
                    if rainbow_index == 6:
                        rainbow_index = 0
                return print(newtxt)
            except err as err:
                print(str(err))
                exit()
        else:
            try:
                if color != None:
                    if color.capitalize() not in colors:
                        raise ValueError("\033[1;31m[ERROR]:\033[0m The color specified does not exist.")
                    else:
                        colorIndex = colors.index(color.capitalize())
                        code = colorcodes[colorIndex]

                if style != None:
                    if style.capitalize() not in styles:
                        raise ValueError("\033[1;31m[ERROR]:\033[0m The style specified does not exist.")
                    else:
                        if style == "Bold":
                            code = code.replace("[0", "[1")
                        elif style == "Dim":
                            code = code.replace("[0", "[2")
                        elif style == "Italicize":
                            code = code.replace("[0", "[3")
                        elif style == "Underline":
                            code = code.replace("[0", "[4")

                if bg_color != None:
                    if bg_color.capitalize() not in colors:
                        raise ValueError("\033[1;31m[ERROR]:\033[0m The background color specified does not exist.")
                    else:
                        if bg_color == "Gray":
                            code = code.replace("48m", "40m")
                        elif bg_color == "Red":
                            code = code.replace("48m", "41m")
                        elif bg_color == "Green":
                            code = code.replace("48m", "42m")
                        elif bg_color == "Yellow":
                            code = code.replace("48m", "43m")
                        elif bg_color == "Blue":
                            code = code.replace("48m", "44m")
                        elif bg_color == "Purple":
                            code = code.replace("48m", "45m")
                        elif bg_color == "Cyan":
                            code = code.replace("48m", "46m")
                        elif bg_color == "White":
                            code = code.replace("48m", "47m")
                
                return print(code + text + endcode)
            except ValueError as err:
                print(str(err))
                exit()
    else:
        try:
            if style != None:
                if style.capitalize() not in styles:
                    raise ValueError("\033[1;31m[ERROR]:\033[0m The style specified does not exist.")
                else:
                    if style == "Bold":
                        code = code.replace("[0", "[1")
                    elif style == "Dim":
                        code = code.replace("[0", "[2")
                    elif style == "Italicize":
                        code = code.replace("[0", "[3")
                    elif style == "Underline":
                        code = code.replace("[0", "[4")

            if bg_color != None:
                if bg_color.capitalize() not in colors:
                    raise ValueError("\033[1;31m[ERROR]:\033[0m The background color specified does not exist.")
                else:
                    if bg_color == "Gray":
                        code = code.replace("48m", "40m")
                    elif bg_color == "Red":
                        code = code.replace("48m", "41m")
                    elif bg_color == "Green":
                        code = code.replace("48m", "42m")
                    elif bg_color == "Yellow":
                        code = code.replace("48m", "43m")
                    elif bg_color == "Blue":
                        code = code.replace("48m", "44m")
                    elif bg_color == "Purple":
                        code = code.replace("48m", "45m")
                    elif bg_color == "Cyan":
                        code = code.replace("48m", "46m")
                    elif bg_color == "White":
                        code = code.replace("48m", "47m")
        
            return print(code + text + endcode)
        except ValueError as err:
            print(str(err))
            exit()
            


def warnUserPrint(text: str) -> str:
    """
    
    Usage
    -----
    Acts like a normal print statement but with a warning notice before the warning text in a yellow color setting.\n
    An example statement would be like so:\n
    [WARNING]: This is an informational message that can let users know about a situation regarding something that can go poetentially wrong.\n
    (The [WARNING]: is already added to the message.)

    Parameters
    ----------
    text : string
        The text you want to be typed out.
    
    Returns
    -------
        Returns out the finished product into the terminal.
        
    """
    code = "\033[1;33m"
    endcode = "\033[0m"
    print(code + f"[WARNING]: {text}{endcode}")



def informUserPrint(text: str) -> str:
    """
    
    Usage
    -----
    Acts like a normal print statement but with a informational notice before the informational text in a gray color setting.\n
    An example statement would be like so:\n
    [INFO]: This is an informational message that can let users know about a situation regarding something that the user must know.\n
    (The [INFO]: is already added to the message.)

    Parameters
    ----------
    text : string
        The text you want to be typed out.
    
    Returns
    -------
        Returns out the finished product into the terminal.
        
    """
    code = "\033[1;30m"
    endcode = "\033[0m"
    print(code + f"[INFO]: {text}{endcode}")



from setuptools import find_packages, setup
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='printextent',
    version='0.1.11',
    packages=find_packages(),
    install_requires=[],
    description='Printing made for the creativity.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Thomas Kyrouac',
    author_email='thomaskkyrouac@gmail.com',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)


def asciiPrint(art: list):
    """

    Usage
    -----
    Prints out ascii art created by the user.\n
    (Each index of the list would be one row of the art piece.)

    Parameters
    ----------
    art : list
        The ascii art you want to print out.

    Returns
    -------
        Returns out the finished product into the terminal.
        
    """

    for line in art:
        print(line)
